#ifndef __PWM_CPP__
#define __PWM_CPP__

#include "pwm.h"
#include "esp_log.h"

/*
  DEV By IOXhop : www.ioxhop.com
  Sonthaya Nongnuch : fb.me/maxthai
*/

pwm::pwm() {
	this->nextPointPWMConfigState = 0;
}

void pwm::init(void) {
	// Config Timer2
	ledc_timer_config_t timer_conf;
	timer_conf.duty_resolution = LEDC_TIMER_16_BIT;
	timer_conf.freq_hz = 500; // 500Hz
	timer_conf.speed_mode = LEDC_HIGH_SPEED_MODE;
	timer_conf.timer_num = LEDC_TIMER_0;
	ledc_timer_config(&timer_conf);
	
	// set initialized flag
	this->initialized = true;
	// clear error flag
	this->error = false;
	
	esp_log_level_set("*", ESP_LOG_INFO);
}

int pwm::prop_count(void) {
	// not supported
	return 0;
}

bool pwm::prop_name(int index, char *name) {
	// not supported
	return false;
}

bool pwm::prop_unit(int index, char *unit) {
	// not supported
	return false;
}

bool pwm::prop_attr(int index, char *attr) {
	// not supported
	return false;
}

bool pwm::prop_read(int index, char *value) {
	// not supported
	return false;
}

bool pwm::prop_write(int index, char *value) {
	// not supported
	return false;
}

void pwm::process(Driver *drv) {
	
}
// --------------------------------------

// Start here
// Method
uint8_t pwm::getIndexFromPin(uint8_t pin) {
	uint8_t index = getLEDCChannelFromPin(pin);
	if (index == 255) {
		index = setupLEDCFromPin(pin);
	}
	return index;
}

uint8_t pwm::getLEDCChannelFromPin(uint8_t pin) {
	uint8_t index = 255;
	for (int i=0;i<nextPointPWMConfigState;i++) {
		if (pwmConfigState[i].pin == pin) {
			index = i;
			break;
		}
	}
	return index;
}

uint8_t pwm::setupLEDCFromPin(uint8_t pin) {
	uint8_t index = nextPointPWMConfigState++;
	
	struct configState &config = pwmConfigState[index];

	config = {
		.pin = pin,
		.ledc_ch = index,
	};
	
	ledc_channel_config_t ledc_conf;
	ledc_conf.channel = static_cast<ledc_channel_t>(config.ledc_ch);
	ledc_conf.duty = 0;
	ledc_conf.gpio_num   = config.pin;
	ledc_conf.intr_type  = LEDC_INTR_DISABLE;
	ledc_conf.speed_mode = LEDC_HIGH_SPEED_MODE;
	ledc_conf.timer_sel  = LEDC_TIMER_0;
	ledc_channel_config(&ledc_conf);
	
	return index;
}

void pwm::setDuty(uint8_t pin, double duty) {
	if (duty > 100.0) duty = 100.0;
	if (duty < 0.0) duty = 0.0;
	
	uint8_t index = getIndexFromPin(pin);
	
	uint16_t realDuty = duty * (double)0xFFFF / 100.0;
	
	ledc_set_duty(LEDC_HIGH_SPEED_MODE, static_cast<ledc_channel_t>(pwmConfigState[index].ledc_ch), realDuty);
	ledc_update_duty(LEDC_HIGH_SPEED_MODE, static_cast<ledc_channel_t>(pwmConfigState[index].ledc_ch));
}

#endif
